package example.controller;

import example.model.User;
import example.utils.DBUtil;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@WebServlet("/userList")
public class UserListServlet extends HttpServlet {
    private static final long serialVersionUID = 1L;
    private static final int PAGE_SIZE = 3; // 每页显示的记录数

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        int currentPage = 1; // 当前页码，默认第一页
        String pageParam = request.getParameter("page");
        if (pageParam!= null &&!pageParam.isEmpty()) {
            currentPage = Integer.parseInt(pageParam);
        }

        String queryParam = request.getParameter("query"); // 获取查询条件参数
        if (queryParam == null) {
            queryParam = ""; // 设置默认值为空字符串，可根据实际情况调整
        }
        List<User> userList = new ArrayList<>();

        String sql = getSqlWithQuery(queryParam); // 根据查询条件获取对应的SQL语句
        int offset = (currentPage - 1) * PAGE_SIZE;
        ResultSet resultSet = DBUtil.executeQuery(sql, offset, PAGE_SIZE);
        try {
            while (resultSet.next()) {
                User user = new User();
                user.setId(resultSet.getInt("id"));
                user.setUsername(resultSet.getString("username"));
                user.setPassword(resultSet.getString("password"));
                userList.add(user);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DBUtil.closeResources(null, null, resultSet);
        }

        int totalCount = getTotalCount(queryParam); // 根据查询条件获取总记录数
        int totalPages = (totalCount + PAGE_SIZE - 1) / PAGE_SIZE; // 计算总页数

        String pageLinks = generatePageLinks(currentPage, totalPages, queryParam); // 生成带查询条件的页码链接

        request.setAttribute("userList", userList);
        request.setAttribute("currentPage", currentPage);
        request.setAttribute("totalPages", totalPages);
        request.setAttribute("pageLinks", pageLinks);
        request.setAttribute("query", queryParam); // 将查询条件也设置为属性传递给JSP页面，用于回显等
        request.getRequestDispatcher("/userList.jsp").forward(request, response);
    }

    private String getSqlWithQuery(String queryParam) {
        StringBuilder sqlBuilder = new StringBuilder("SELECT * FROM user");
        if (queryParam!= null &&!queryParam.isEmpty()) {
            sqlBuilder.append(" WHERE username LIKE '%").append(queryParam).append("%'"); // 简单示例，按用户名模糊查询
        }
        sqlBuilder.append(" LIMIT ?,?");
        return sqlBuilder.toString();
    }

    private int getTotalCount(String queryParam) {
        StringBuilder sqlBuilder = new StringBuilder("SELECT COUNT(*) FROM user");
        if (queryParam!= null &&!queryParam.isEmpty()) {
            sqlBuilder.append(" WHERE username LIKE '%").append(queryParam).append("%'"); // 按用户名模糊查询统计数量
        }
        ResultSet resultSet = DBUtil.executeQuery(sqlBuilder.toString());
        int totalCount = 0;
        try {
            if (resultSet.next()) {
                totalCount = resultSet.getInt(1);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            DBUtil.closeResources(null, null, resultSet);
        }
        return totalCount;
    }

    private String generatePageLinks(int currentPage, int totalPages, String queryParam) {
        StringBuilder pageLinksBuilder = new StringBuilder();
        pageLinksBuilder.append("<ul class='pagination'>");
        for (int i = 1; i <= totalPages; i++) {
            if (i == currentPage) {
                pageLinksBuilder.append("<li class='page-item active'><a class='page-link' href='#'>").append(i).append("</a></li>");
            } else {
                String queryParamEncoded;
                if (queryParam == null) {
                    queryParamEncoded = ""; // 如果查询条件为null，赋空字符串作为默认值
                } else {
                    try {
                        queryParamEncoded = java.net.URLEncoder.encode(queryParam, "UTF-8"); // 对查询条件进行URL编码
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                        queryParamEncoded = ""; // 编码出现异常也赋空字符串，避免后续问题
                    }
                }
                pageLinksBuilder.append("<li class='page-item'><a class='page-link' href='userList?page=").append(i).append("&query=").append(queryParamEncoded).append("'>").append(i).append("</a></li>");
            }
        }
        pageLinksBuilder.append("</ul>");
        return pageLinksBuilder.toString();
    }
}